@@ -2,10 +2,14 @@
 
 """Module for the parsing functions and objects."""
 
+from __future__ import annotations
 from enum import Enum, auto
 import re
 from typing import Any, Callable, Mapping, Tuple, Union, Dict
 
+import loguru
+from loguru import logger
+
 from .exceptions import ParseError, NotEnoughDataError, TooMuchDataError
 from .header_values import (
     BytesHeaderValue,
@@ -61,11 +65,16 @@ class Parser:
         self._state = start
         self.buffer = b""
 
-        self._state_table = {
-            States.Status: self.status,
-            States.Header: self.header,
-            States.Body: self.body,
-        }
+        self._logger = logger
+
+    def _bind(self, **kwargs) -> loguru.Logger:
+        """Helper method to bind a new logger.
+
+        :return: A new logger instance.
+        """
+
+        self._logger = self._logger.bind(**kwargs)
+        return self._logger
 
     @property
     def state(self) -> States:
@@ -113,8 +122,11 @@ class Parser:
 
         if status_line and delimiter:
             self.buffer = leftover
-            self.result = {**self.result, **self.status_parser(status_line)}
+            parsed_status = self.status_parser(status_line)
+            self.result = {**self.result, **parsed_status}
             self._state = States.Header
+            self._bind(**parsed_status)
+            self._logger.debug("Finished parsing status line")
         else:
             raise NotEnoughDataError
 
@@ -141,21 +153,80 @@ class Parser:
 
         header_line, delimiter, leftover = self.buffer.partition(self.delimiter)
 
-        if not header_line and delimiter and leftover == self.delimiter:
+        if self._at_end_of_headers_with_empty_body(header_line, delimiter, leftover):
             self.buffer = b""
             self._state = States.Body
-        elif not header_line and delimiter:
+            self._bind(headers=self.result["headers"])
+            self._logger.debug("Finished parsing headers")
+        elif self._at_end_of_headers(header_line, delimiter):
             self.buffer = leftover
             self._state = States.Body
-        elif header_line and delimiter:
+            self._bind(headers=self.result["headers"])
+            self._logger.debug("Finished parsing headers")
+        elif self._at_next_header(header_line, delimiter):
             self.buffer = leftover
             key, value = self.header_parser(header_line)
             self.result["headers"][key] = value
-        elif not header_line and not delimiter and not leftover:
+            self._logger.debug("Parsed header {}", key)
+        elif self._is_status_line_only(header_line, delimiter, leftover):
             self._state = States.Body
+            self._logger.debug("No headers to parse")
         else:
             raise ParseError("Header section not in recognizable format")
 
+    def _at_end_of_headers_with_empty_body(
+        self, header_line: bytes, delimiter: bytes, leftover: bytes
+    ) -> bool:
+        """Helper method to check if the header sections is done and there is an empty body.
+
+        :param header_line: Contents of the header line.
+        :param delimiter: Separator between the header line and the rest of the response.
+        :param leftover: Remaining response contents.
+
+        :return: If the header section is finished, and there is no remaining content for a body.
+        """
+
+        return all([not header_line, delimiter, leftover == self.delimiter])
+
+    @staticmethod
+    def _at_end_of_headers(header_line: bytes, delimiter: bytes) -> bool:
+        """Helper method to check if the parser is at the end of the header section.
+
+        :param header_line: Contents of the header line.
+        :param delimiter: Separator between the header line and the rest of the response.
+
+        :return: If the header section is finished and there is a delimiter.
+        """
+
+        return all([not header_line, delimiter])
+
+    @staticmethod
+    def _at_next_header(header_line: bytes, delimiter: bytes) -> bool:
+        """Helper method to check if a header can be parsed.
+
+        :param header_line: Contents of the header line.
+        :param delimiter: Separator between the header line and the rest of the response.
+
+        :return: If the header line is finished, and there is another header.
+        """
+
+        return all([header_line, delimiter])
+
+    @staticmethod
+    def _is_status_line_only(
+        header_line: bytes, delimiter: bytes, leftover: bytes
+    ) -> bool:
+        """Helper method to check if the response is a status line only, without a header section.
+
+        :param header_line: Contents of the header line.
+        :param delimiter: Separator between the header line and the rest of the response.
+        :param leftover: Remaining response contents.
+
+        :return: If the response was a status line only without a header section or body.
+        """
+
+        return all([not header_line, not delimiter, not leftover])
+
     def body(self) -> None:
         """Uses the length defined in the `Content-length` header (defaulted to 0) to determine how many bytes the body
         contains.
@@ -173,8 +244,10 @@ class Parser:
         try:
             self.result["body"] += self.body_parser(self.buffer, content_length)
             self._state = States.Done
+            self._logger.debug("Finished parsing body")
         except TooMuchDataError:
             self._state = States.Done
+            self._logger.error("Body parsed was larger than the content length")
             raise
 
 
@@ -230,7 +303,7 @@ def parse_response_status(stream: bytes) -> Dict[str, Union[str, int]]:
 
     try:
         protocol_version, status_code, message = list(
-            filter(None, stream.decode("ascii").split(" "))
+            filter(None, stream.decode("ascii").split(" ", 2))
         )
         protocol, version = protocol_version.split("/")
     except ValueError as error:
@@ -376,6 +449,13 @@ def parse_spam_value(stream: str) -> SpamValue:
 
 
 def parse_user_value(stream: str) -> UserValue:
+    """Parse the username.
+
+    :param stream: String of username to parse. Whitespace is trimmed.
+
+    :return: The `UserValue` instance.
+    """
+
     return UserValue(name=stream.strip())
 
 
@@ -459,6 +539,8 @@ class RequestParser(Parser):
     """Sub-class of the parser for requests."""
 
     def __init__(self):
+        """RequestParse constructor."""
+
         super().__init__(
             delimiter=b"\r\n",
             status_parser=parse_request_status,
@@ -471,6 +553,8 @@ class ResponseParser(Parser):
     """Sub-class of the parser for responses."""
 
     def __init__(self):
+        """ResponseParser constructor."""
+
         super().__init__(
             delimiter=b"\r\n",
             status_parser=parse_response_status,
